#include <stdio.h>
#include <stdlib.h>
#include <string.h>

struct int_node {
  int val;
  int height;
  struct int_node *left;  
  struct int_node *right;
};

typedef struct int_node * tree;

int get_height(tree t) {
  if (t == NULL)
    return 0;
  else
    return t->height;
}

void fix_height(tree t) {
  int h = get_height(t->left);
  if (get_height(t->right) > h)
    h = get_height(t->right);
  t->height = h + 1;
}

tree leaf(int n) {
  tree t = (tree)malloc(sizeof(struct int_node));

  t->val = n;
  t->left = NULL;
  t->right = NULL;
  t->height = 1;

  return t;
}

tree branch(int n, tree l, tree r) {
  tree t = (tree)malloc(sizeof(struct int_node));

  t->val = n;
  t->left = l;
  t->right = r;
  fix_height(t);

  return t;
}

int get_balance(tree t) {
  return get_height(t->left) - get_height(t->right);
}

void print_tree(tree t, int indent) {
  if (t != NULL) {
    int i;

    print_tree(t->right, indent + 2);

    for (i = 0; i < indent; i++)
      printf(" ");
    printf("- %d\n", t->val);

    print_tree(t->left, indent + 2);
  }
}

/****************************************/

int btsearch(tree t, int val)
{
  while (t != NULL) {
    if (t->val > val)
      t = t->left;
    else if (t->val < val)
      t = t->right;
    else
      return 1;
  }
  return 0;
}

/****************************************/

tree btinsert_left(tree t, int val);
tree btinsert_right(tree t, int val);

tree btinsert(tree t, int val)
{
  if (t == NULL)
    return leaf(val);
  else {
    if (t->val > val)
      return btinsert_left(t, val);
    else if (t->val < val)
      return btinsert_right(t, val);
    else
      return t;
  }
}

tree btinsert_left(tree t, int val) {
  /* insert smaller number to left */
  t->left = btinsert(t->left, val);
  fix_height(t);

  if (get_balance(t) == 2) {
    /* need to rotate right */
    tree left = t->left;
    if (get_balance(left) < 0) {
      /* double right rotation */
      tree left_right = left->right;
      left->right = left_right->left;
      left_right->left = left;
      fix_height(left);
      left = left_right;
    }
    t->left = left->right;
    left->right = t;
    fix_height(t);
    fix_height(left);
    return left;
  }
   
  return t;
}

tree btinsert_right(tree t, int val) {
  /* insert larger number to right */
  t->right = btinsert(t->right, val);
  fix_height(t);

  if (get_balance(t) == -2) {
    /* need to rotate left */
    tree right = t->right;
    if (get_balance(right) > 0) {
      /* double left rotation */
      tree right_left = right->left;
      right->left = right_left->right;
      right_left->right = right;
      fix_height(right);
      right = right_left;
    }
    t->right = right->left;
    right->left = t;
    fix_height(t);
    fix_height(right);
    return right;
  }
  
  return t;
}

/****************************************/
/* Tests */

void failed(int a, int b) {
  printf("test failed; expected %d, got %d\n", b, a);
  exit(1);
}

void check_int(int a, int b) {
  if (a != b) failed(a, b);
}

void test() {
  tree t = NULL;

  t = btinsert(t, 5);
  t = btinsert(t, 1);
  t = btinsert(t, 3);
  t = btinsert(t, 2);
  t = btinsert(t, 4);
  t = btinsert(t, 10);
  t = btinsert(t, 7);
  t = btinsert(t, 6);
  t = btinsert(t, 11);

  check_int(btsearch(t, 0), 0);
  check_int(btsearch(t, 1), 1);
  check_int(btsearch(t, 2), 1);
  check_int(btsearch(t, 3), 1);
  check_int(btsearch(t, 4), 1);
  check_int(btsearch(t, 5), 1);
  check_int(btsearch(t, 6), 1);
  check_int(btsearch(t, 7), 1);
  check_int(btsearch(t, 8), 0);
  check_int(btsearch(t, 9), 0);
  check_int(btsearch(t, 10), 1);
  check_int(btsearch(t, 11), 1);
  check_int(btsearch(t, 12), 0);

  t = btinsert(t, 8);
  check_int(btsearch(t, 8), 1);
  check_int(btsearch(t, 7), 1);
  check_int(btsearch(t, 9), 0);
  check_int(btsearch(t, 10), 1);

  t = btinsert(t, -1);
  check_int(btsearch(t, -1), 1);
  check_int(btsearch(t, 0), 0);
  check_int(btsearch(t, 1), 1);

  t = btinsert(t, 12);
  check_int(btsearch(t, 12), 1);
  check_int(btsearch(t, 13), 0);
  check_int(btsearch(t, 11), 1);
}

/****************************************/

int main(int argc, char** argv)
{
  tree t;
  char buffer[256];
  int i, print_it;
  FILE *f;

  test();

  t = NULL;

  if (!strcmp(argv[1], "-")) {
    /* accept tree from stdin and print it */
    f = stdin;
    print_it = 1;
  } else {
    /* read tree from a file and don't print */
    f = fopen(argv[1], "r");
    print_it = 0;
  }

  while (fgets(buffer, 256, f)) {
    t = btinsert(t, atoi(buffer));
    if (print_it)
      print_tree(t, 0);
  }
  fclose(f);

  while (fgets(buffer, 256, stdin)) {
    i = btsearch(t, atoi(buffer));
    if (i == 0)
      printf("Not there\n");
    else
      printf("There\n");
  }

  return 0;
}
