#include <stdio.h>
#include <stdlib.h>

struct int_node {
  int val;
  struct int_node *left;  
  struct int_node *right;
};

typedef struct int_node * tree;

tree leaf(int n) {
  tree t = (tree)malloc(sizeof(struct int_node));

  t->val = n;
  t->left = NULL;
  t->right = NULL;

  return t;
}

tree branch(int n, tree l, tree r) {
  tree t = (tree)malloc(sizeof(struct int_node));

  t->val = n;
  t->left = l;
  t->right = r;

  return t;
}

/****************************************/

int btsearch_recur(tree t, int val)
{
  if (t == NULL)
    return 0;
  else {
    if (t->val > val)
      return btsearch_recur(t->left, val);
    else if (t->val < val)
      return btsearch_recur(t->right, val);
    else
      return 1;
  }
}

int btsearch(tree t, int val)
{
  while (t != NULL) {
    if (t->val > val)
      t = t->left;
    else if (t->val < val)
      t = t->right;
    else
      return 1;
  }
  return 0;
}

/****************************************/

tree btinsert_recur(tree t, int val)
{
  if (t == NULL)
    return leaf(val);
  else {
    if (t->val > val)
      return branch(t->val,
                    btinsert_recur(t->left, val),
                    t->right);
    else if (t->val < val)
      return branch(t->val,
                    t->left,
                    btinsert_recur(t->right, val));
    else
      return t;
  }
}

tree btinsert(tree t, int val)
{
  tree top = t;
  tree prev = NULL;
  int prev_side = 0;

  while (t != NULL) {
    prev = t;
    if (t->val > val) {
      prev_side = 0;
      t = t->left;
    } else if (t->val < val) {
      prev_side = 1;
      t = t->right;
    } else
      return top;
  }

  t = leaf(val);
  if (prev) {
    if (prev_side == 0)
      prev->left = t;
    else
      prev->right = t;
  }

  if (top)
    return top;
  else
    return t;
}

/****************************************/
/* Tests */

void failed(int a, int b) {
  printf("test failed; expected %d, got %d\n", b, a);
  exit(1);
}

void check_int(int a, int b) {
  if (a != b) failed(a, b);
}

void test() {
  tree t = branch(5,
                  branch(3,
                         branch(1, NULL, leaf(2)),
                         leaf(4)),
                  branch(10,
                         branch(7, leaf(6), NULL),
                         leaf(11)));

  check_int(btsearch(t, 0), 0);
  check_int(btsearch(t, 1), 1);
  check_int(btsearch(t, 2), 1);
  check_int(btsearch(t, 3), 1);
  check_int(btsearch(t, 4), 1);
  check_int(btsearch(t, 5), 1);
  check_int(btsearch(t, 6), 1);
  check_int(btsearch(t, 7), 1);
  check_int(btsearch(t, 8), 0);
  check_int(btsearch(t, 9), 0);
  check_int(btsearch(t, 10), 1);
  check_int(btsearch(t, 11), 1);
  check_int(btsearch(t, 12), 0);

  t = btinsert(t, 8);
  check_int(btsearch(t, 8), 1);
  check_int(btsearch(t, 7), 1);
  check_int(btsearch(t, 9), 0);
  check_int(btsearch(t, 10), 1);

  t = btinsert(t, -1);
  check_int(btsearch(t, -1), 1);
  check_int(btsearch(t, 0), 0);
  check_int(btsearch(t, 1), 1);

  t = btinsert(t, 12);
  check_int(btsearch(t, 12), 1);
  check_int(btsearch(t, 13), 0);
  check_int(btsearch(t, 11), 1);
}

/****************************************/

int main(int argc, char** argv)
{
  tree t;
  char buffer[256];
  int i;
  FILE *f;

  test();

  t = NULL;

  f = fopen(argv[1], "r");
  while (fgets(buffer, 256, f)) {
    t = btinsert(t, atoi(buffer));
  }
  fclose(f);

  while (fgets(buffer, 256, stdin)) {
    i = btsearch(t, atoi(buffer));
    if (i == 0)
      printf("Not there\n");
    else
      printf("There\n");
  }

  return 0;
}
